# -*- coding: utf-8 -*-


class GOSTaskException(Exception):
    pass


class GOSExecutableContainerException(Exception):
    pass


class GOSIOException(Exception):
    pass


class GOSCriticalException(Exception):
    pass
